"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetZosLog = void 0;
const core_for_zowe_sdk_1 = require("@zowe/core-for-zowe-sdk");
const GetZosLogValidator_1 = require("./GetZosLogValidator");
const ZosLogConstants_1 = require("./ZosLogConstants");
const util_1 = require("util");
/**
 * Get zos log via z/OSMF restful api
 * @export
 * @class GetZosLog
 */
class GetZosLog {
    /**
     * Issue an z/OSMF log command, returns "raw" z/OSMF response
     * @static
     * @param {AbstractSession} session representing connection to this api
     * @param {IZosLogParms} commandParms log api parameters, @see {IZosLogParms}
     * @return {Promise<IZosLogType>} command response , @see {IZosLogType}
     * @memberof GetLog
     */
    static getZosLog(session, commandParms) {
        return __awaiter(this, void 0, void 0, function* () {
            GetZosLogValidator_1.GetZosLogValidator.validateSession(session);
            let resp = yield core_for_zowe_sdk_1.ZosmfRestClient.getExpectJSON(session, GetZosLog.getResourcePath(commandParms), [core_for_zowe_sdk_1.ZosmfHeaders.X_CSRF_ZOSMF_HEADER]);
            if (util_1.isNullOrUndefined(commandParms.processResponses) || commandParms.processResponses !== false) {
                // the IBM responses sometimes have \r and sometimes \r\n, we will process them our here and hopefully
                // return them with just \n.
                resp = JSON.parse(JSON.stringify(resp).replace(/\\r\\n/g, "\\n").replace(/\\r/g, "\\n"));
            }
            return resp;
        });
    }
    /**
     * Get resource path for z/OSMF log restful api
     * @static
     * @param {IZosLogParms} commandParms params to compose the resource path
     * @return {string} resource path
     * @memberof GetCommand
     */
    static getResourcePath(commandParms) {
        let path = ZosLogConstants_1.ZosLogConstants.RESOURCE;
        if (commandParms.startTime !== undefined) {
            let startTime = commandParms.startTime;
            // in case the input is milliseconds format, which is a long number
            if (!isNaN(commandParms.startTime)) {
                startTime = new Date(+startTime).toISOString();
            }
            path += "time=" + startTime + "&";
        }
        if (commandParms.direction !== undefined) {
            path += "direction=" + commandParms.direction + "&";
        }
        if (commandParms.range !== undefined) {
            path += "timeRange=" + commandParms.range + "&";
        }
        return path;
    }
}
exports.GetZosLog = GetZosLog;
//# sourceMappingURL=GetZosLog.js.map